
/*  -*- MaTX -*-
 *
 *  【名前】
 *      diag_vec() - ベクトルと対角行列の変換
 *
 *  【形式】
 *      Y = diag_vec(X)
 *         (Matrix|Array) Y;
 *         (Matrix|Array) X;
 *
 *      Y = diag_vec(X,k)
 *         (Matrix|Array) Y;
 *         (Matrix|Array) X;
 *         Integer k;
 *
 *  【機能説明】
 *      X がベクトルのとき，diag_vec(X) は vec2diag(X) に等しく，
 *      X が行列のとき，diag_vec(X) は diag2vec(X) に等しい。
 *
 *      X がベクトルのとき，diag_vec(X,k) は vec2diag(X,k) に等しく，
 *      X が行列のとき，diag_vec(X,k) は diag2vec(X,k) に等しい。
 *
 *  【例題】
 *      >> Y = diag_vec([[1 2 3][4 5 6][7 8 9]])
 *      === [Y] : (  3,  1) ===
 *                 (  1)     
 *      (  1)  1.00000000E+00
 *      (  2)  5.00000000E+00
 *      (  3)  9.00000000E+00
 *      >> Y = diag_vec([1 5 9])
 *      === [Y] : (  3,  3) ===
 *                 (  1)           (  2)           (  3)     
 *      (  1)  1.00000000E+00  0.00000000E+00  0.00000000E+00
 *      (  2)  0.00000000E+00  5.00000000E+00  0.00000000E+00
 *      (  3)  0.00000000E+00  0.00000000E+00  9.00000000E+00
 *
 *  【関連項目】
 *      diag, vec2diag, diag2vec
 */

Func Matrix diag_vec(A_, k, ...)
	Matrix A_;
	Integer k;
{
	Integer n;
	Matrix AA, A, B;
	
	error(nargchk(1, 2, nargs, "diag_vec"));
	
	if (nargs == 1) {
		k = 0;
	}
	
	if (length(A_) == 0) {
		return [];
	}

	A = A_;
	
	if (Rows(A) == 1 || Cols(A) == 1) {
		B = vec2diag(A);
		n = length(A);
		if (k > 0) {
			AA = [[Z(n,k), B]
				  [Z(k,n+k)]];
		} else if (k < 0) {
			AA = [[Z(abs(k),n+abs(k))]
				  [B, Z(n,abs(k))]];
		} else {
			AA = B;
		}
	} else {
		if (Rows(A) < Cols(A)) {
			A = A(1:Rows(A),1:Rows(A));
		} else if (Rows(A) > Cols(A)) {
			A = A(1:Cols(A),1:Cols(A));
		}

		n = Rows(A);
		if (n-1 < abs(k)) {
			error("diag_vec():  指数が行列の大きさより大きい。\n");
		}
		if (k > 0) {
			B = A(1:n-k,k+1:n);
		} else if (k < 0) {
			B = A(abs(k)+1:n,1:n-abs(k));
		} else {
			B = A;
		}
		AA = diag2vec(B);
	}

	return AA;
}
